// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSPointerController.m
//  PointingScrollSimulator
//

#import "TSPointerController.h"

#import <ApplicationServices/ApplicationServices.h>
#import <dispatch/dispatch.h>

typedef enum {
    TSDeviceIPhone,
    TSDeviceIPhoneRetina,
    TSDeviceIPad,
    TSDeviceIPadRetina,
    TSDeviceUnknown
} TSDeviceType;

// Margin at the top and bottom of the screen to avoid starting too close to 
// the bottom edge, or hitting the status bar.
const CGFloat kMinimumScreenMargin = 50.f;

// Interval between mouse events (ns). If this is too small, the simulator
// doesn't capture get all of them and UIScrollView's flick detection becomes
// spotty.
const uint64_t kMinimumEventInterval = 10 * NSEC_PER_MSEC;

/**
 * Returns the type of simulator window that has the given screen rect.
 */
static TSDeviceType _devicetype(CGRect bounds) 
{
    switch ((int)bounds.size.width) {
        case 368: return TSDeviceIPhone;
        case 724: return TSDeviceIPhoneRetina;
        case 768: return TSDeviceIPad;
        case 1551: return TSDeviceIPadRetina;
        default: return TSDeviceUnknown;
    }
}

/**
 * Returns the rect of the simulator device's screen that is defined by the
 * given simulator bounds.
 */
static CGRect _deviceScreenBounds(CGRect bounds) 
{
    CGPoint origin = bounds.origin;
    switch (_devicetype(bounds)) {
        case TSDeviceIPhone: return CGRectMake(origin.x + 25, origin.y + 120, 320, 480);
        case TSDeviceIPad: return CGRectMake(origin.x + 25, origin.y + 0, 768 , 1024);
        default: return CGRectZero;
    }
}

/**
 * Creates a dispatch timer.
 * Interval is measured in nanoseconds. Use NSEC_PER_[M]SEC to convert.
 */
static dispatch_source_t _createDispatchTimer(dispatch_queue_t queue,
                                              uint64_t interval,
                                              dispatch_block_t block)
{
    dispatch_source_t timer = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER,
                                                     0, 0, queue);
    if (timer) {
        dispatch_source_set_timer(timer, dispatch_time(DISPATCH_TIME_NOW, 0),
                                  interval, 0);
        dispatch_source_set_event_handler(timer, block);
        dispatch_resume(timer);
    }
    return timer;    
}

@interface TSPointerController ()
// Timer used to scan for simulator instances
@property (strong) NSTimer *scanTimer;
@property (strong) NSRunningApplication *simulatorApp;
@end

@implementation TSPointerController
@synthesize window = _window;
@synthesize scrollButton = _startScrolling;
@synthesize scanTimer = _scanTimer;
@synthesize simulatorApp = _simulatorApp;
@synthesize connectionTextView = _simulatorConnection;
@synthesize strokes = _strokes;
@synthesize strokeLength = _strokeLength;
@synthesize strokeVelocity = _strokeDuration;
@synthesize strokeDirection = _strokeDirection;
@synthesize strokeInterval = _strokeInterval;

#pragma mark Init & Dealloc
-(void)awakeFromNib
{
    self.strokes = 1;
    self.strokeLength = 300;
    self.strokeVelocity = 0.6f;
    self.strokeDirection = TSStrokeDown;
    self.strokeInterval = 500;
    
    [self scanForSimulator:self];
    self.scanTimer = [NSTimer scheduledTimerWithTimeInterval:1.f
                                                      target:self
                                                    selector:@selector(scanForSimulator:) 
                                                    userInfo:nil
                                                     repeats:YES];
}

-(void)dealloc
{
    [self.scanTimer invalidate];
}

#pragma mark Simulator Scanning

-(IBAction)scanForSimulator:(id)sender
{
    CGRect window = CGRectZero;
    // Look for the simulator app
    NSArray *apps = [NSRunningApplication runningApplicationsWithBundleIdentifier:@"com.apple.iphonesimulator"];
    if ([apps count] == 1) {
        self.simulatorApp = [apps objectAtIndex:0];
        CFArrayRef windows = CGWindowListCreate(kCGWindowListOptionAll, kCGNullWindowID);
        NSArray *descriptions = (__bridge_transfer NSArray*)CGWindowListCreateDescriptionFromArray(windows);
        CFRelease(windows);
        
        // Find the window containing the actual device simulation.
        // The simulator has several windows; the one we're interested in has a
        // name that contains "Simulator".
        // HACK: Probably not true for localised versions.
        // TODO: Doesn't work for retina; use accessibility api
        for (NSDictionary* dict in descriptions) {
            NSNumber *pid = [dict objectForKey:(__bridge NSString*)kCGWindowOwnerPID];
            NSString *name = [dict objectForKey:(__bridge NSString*)kCGWindowName];
            if ([pid intValue] == [self.simulatorApp processIdentifier] && 
                name && [name rangeOfString:@"Simulator"].location != NSNotFound) {
                NSDictionary* bounds = [dict objectForKey:(__bridge NSString*)kCGWindowBounds];
                CGRectMakeWithDictionaryRepresentation((__bridge CFDictionaryRef)bounds, &window);
                break;
            }
        }
    }
    
    if (CGRectIsEmpty(window)) {
        // Can't find simulator
        _simulatorFrame = CGRectZero;
        self.simulatorApp = nil;
        [self.connectionTextView setStringValue:@"No connection to simulator."];
        [self.connectionTextView setTextColor:[NSColor redColor]];
        [self.scrollButton setEnabled:NO];
    } else {
        // Found simulator
        _simulatorFrame = window;
        [self.connectionTextView setStringValue:@"Connected to simulator."];
        [self.connectionTextView setTextColor:[NSColor greenColor]];
        [self.scrollButton setEnabled:YES];
    }
}

#pragma mark Scrolling
-(IBAction)startScrolling:(id)sender
{
    [self.scrollButton setEnabled:NO];
    [self.simulatorApp activateWithOptions:NSApplicationActivateIgnoringOtherApps];
    [NSThread sleepForTimeInterval:0.1f];
    
    // Calculate the start and target coordinates
    CGRect screenBounds = _deviceScreenBounds(_simulatorFrame);
    CGPoint startPoint = CGPointMake(screenBounds.origin.x + (screenBounds.size.width/2), 0);
    CGPoint targetPoint = startPoint;
    if (self.strokeDirection == TSStrokeDown) {
        startPoint.y = screenBounds.origin.y + screenBounds.size.height - kMinimumScreenMargin;
        targetPoint.y = startPoint.y - self.strokeLength;
    } else {
        startPoint.y = screenBounds.origin.y + kMinimumScreenMargin;
        targetPoint.y = startPoint.y + self.strokeLength;
    }
    
    if (!CGRectContainsPoint(screenBounds, targetPoint)) {
        // Maybe this is alright... flicking to a point outside the screen
        // assert(0);
    }

    // Run on a concurrent dispatch queue, because we block on a semaphore
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0);
    dispatch_async(queue, ^{
        // Displacement of each event
        CGFloat magnitude = (kMinimumEventInterval/NSEC_PER_MSEC) * self.strokeVelocity;
        magnitude *= (self.strokeDirection == TSStrokeDown) ? -1 : 1;
        
        int strokeCount = 0;
        // Signalled when a complete stroke is completed
        dispatch_semaphore_t strokeLock = dispatch_semaphore_create(0);
        while (strokeCount++ != self.strokes) {
            
            __block CGPoint current = startPoint;
            // Move the cursor into position
            CGWarpMouseCursorPosition(current);
            [NSThread sleepForTimeInterval:0.1f];
            CGEventRef event = CGEventCreateMouseEvent(NULL, kCGEventLeftMouseDown, current, 0);
            CGEventPost(kCGHIDEventTap, event);
            CFRelease(event);
            
            // Stroke
            __block dispatch_source_t timer = _createDispatchTimer(queue, kMinimumEventInterval, ^{          
                current.y += magnitude;                
                
                CGEventRef event = CGEventCreateMouseEvent(NULL, kCGEventLeftMouseDragged, current, 0);
                CGEventPost(kCGHIDEventTap, event);
                CFRelease(event);
                
                if ((self.strokeDirection == TSStrokeDown && current.y <= targetPoint.y) ||
                    (self.strokeDirection == TSSStrokeDown && current.y >= targetPoint.y)) {
                    // Wrap up when a stroke is completed
                    
                    [NSThread sleepForTimeInterval:((NSTimeInterval)self.dwellTime)/1000.0];
                    /*
                    current.y += copysign(10, magnitude);
                    event = CGEventCreateMouseEvent(NULL, kCGEventLeftMouseDragged, current, 0);
                    CGEventPost(kCGHIDEventTap, event);
                    CFRelease(event);
                    
                    [NSThread sleepForTimeInterval:0.5f];
                    */
                    event = CGEventCreateMouseEvent(NULL, kCGEventLeftMouseUp, current, 0);
                    CGEventPost(kCGHIDEventTap, event);
                    CFRelease(event);
                    dispatch_suspend(timer);
                    dispatch_semaphore_signal(strokeLock);
                }
            });
            
            // Wait for the next stroke
            dispatch_semaphore_wait(strokeLock, DISPATCH_TIME_FOREVER);
            [NSThread sleepForTimeInterval:((NSTimeInterval)self.strokeInterval)/1000.0];
        }
    });
 }

@end
