// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSScrollViewController.m
//  TouchScrollSimulator
//

#import "TSScrollViewController.h"
#import "PrivateAPI/UIApplication+Extensions.h"
#import "PrivateAPI/UINavigationButton.h"
#import "TSStripedView.h"
#import "TSScrollView.h"
#import "TSEventRecorder.h"
#import "TSLogger.h"

const NSTimeInterval kTSScrollViewResetTimeInterval = 10.f;

// Indicates what the view is being used for
typedef enum {
    TSControllerTest = 0,
    TSControllerPlayback = 1,
    TSControllerRecorder = 2
} TSControllerState;

@interface TSScrollViewController ()
@property (strong) TSEventRecorder *recorder;
@property (assign) TSControllerState state;
@property (strong) NSTimer *resetTimer;
// Label to use if recording
@property (strong) NSString *scriptLabel;

-(void)_back:(id)sender;
-(void)_scrollViewStarted:(NSNotification*)n;
-(void)_scrollViewStopped:(NSNotification*)n;
-(void)_playbackCompleted:(NSDictionary*)info;
-(void)_resetScrollView:(NSTimer*)t;
@end

@implementation TSScrollViewController
@synthesize scriptLabel = _scriptLabel;
@synthesize state = _state;
@synthesize recorder = _recorder;
@synthesize scrollView = _scrollView;
@synthesize contentView = _contentView;
@synthesize shouldAutomaticallyReset = _shouldAutomaticallyReset;
@synthesize resetToTop = _resetToTop;
@synthesize resetTimer = _resetTimer;
@synthesize startAtBottom = _startAtBottom;

//variable for reset at bottom if start at bottom
CGPoint entryPoint;

-(void)viewDidLoad
{
    [super viewDidLoad];
	[self.scrollView setContentSize:self.contentView.bounds.size];
    
    // Create a button that looks like a normal navigation bar back button.
    UINavigationButton *backButton = [[NSClassFromString(@"UINavigationButton") alloc] initWithTitle:@"Back"
                                                                                               style:1];
    backButton.barStyle = UIBarStyleBlackTranslucent;
    backButton.opaque = NO;
    
    [backButton sizeToFit];
    CGPoint p = backButton.center;
    p.x += 10.f; p.y += 10.f;
    backButton.center = p;
    
    [backButton addTarget:self action:@selector(_back:) forControlEvents:UIControlEventTouchUpInside];
    
    [self.view addSubview:backButton];
    
    // Watch for the scroll view motion changes
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_scrollViewStarted:)
                                                 name:TSScrollViewStartedScrollingNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_scrollViewStopped:)
                                                 name:TSScrollViewStoppedScrollingNotification object:nil];
}

-(void)playScriptAtURL:(NSURL*)url
{
    self.state = TSControllerPlayback;
    self.recorder = [[TSEventRecorder alloc] initWithContentsOfURL:url];
}

-(void)recordScriptWithLabel:(NSString*)label
{
    self.state = TSControllerRecorder;
    self.scriptLabel = label;
    self.recorder = [[TSEventRecorder alloc] init];
    [[UIApplication sharedApplication] _addRecorder:self.recorder];
}

-(void)_playbackCompleted:(NSDictionary*)info
{
    NSLog(@"Playback completed");
}

-(void)viewWillAppear:(BOOL)animated
{
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    [[UIApplication sharedApplication] setStatusBarHidden:YES withAnimation:UIStatusBarAnimationSlide];
}

-(void)_back:(id)sender
{
    [self.navigationController popViewControllerAnimated:YES];
}

-(void)_scrollViewStarted:(NSNotification*)n
{
    [self.resetTimer invalidate];
    self.resetTimer = nil;
}

-(void)_scrollViewStopped:(NSNotification*)n
{
    if (self.shouldAutomaticallyReset) {
        self.resetTimer = [NSTimer scheduledTimerWithTimeInterval:kTSScrollViewResetTimeInterval
                                                           target:self
                                                         selector:@selector(_resetScrollView:) userInfo:nil repeats:NO];
    }
}

-(void)_resetScrollView:(NSTimer*)t
{
    if (self.resetToTop) {
        [self.scrollView setContentOffset:entryPoint animated:YES];
    } else {
        [self.scrollView setContentOffset:CGPointMake(0.f, self.scrollView.contentSize.height - self.scrollView.bounds.size.height)
                                 animated:YES];
    }
}

-(void)viewDidAppear:(BOOL)animated
{   
    if (self.state == TSControllerPlayback) {
        [NSThread sleepForTimeInterval:1.f];
        [[UIApplication sharedApplication] _playbackEvents:self.recorder.events
                                            atPlaybackRate:1.f 
                                           messageWhenDone:self withSelector:@selector(_playbackCompleted:)];
    }
    entryPoint = CGPointZero;
    if(_startAtBottom){
        entryPoint = CGPointMake(self.scrollView.contentOffset.x, self.contentView.bounds.size.height-self.scrollView.bounds.size.height);
     [self.scrollView setContentOffset:entryPoint animated:NO];
        NSLog(@"%.3f",entryPoint.y);
    }
}

-(void)viewWillDisappear:(BOOL)animated
{
    if (self.state == TSControllerRecorder) {
        // Remove the events that cancel the recorder (usually pressing the
        // home button).
        [self.recorder removeLastEvent];
        [self.recorder removeLastEvent];
        [self.recorder writeWithLabel:self.scriptLabel];
    }
    
    [[TSLogger sharedLogger] endSession];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [[UIApplication sharedApplication] setStatusBarHidden:NO withAnimation:UIStatusBarAnimationSlide];
}

@end
